/*
/******************************/
/*    ROBOT                   */
/******************************/
#include <cv.h>
#include <highgui.h>
#include <cstdio>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include "tinyxml.h"
#include "tinystr.h"
#include "closedContour.h"
#include "visualization.h"
#include <cstring>

using namespace std;

class ROBOT{
protected:
	IplImage* img;
	std::vector<int> matchFound;
	
	//vector<ClosedContour*> objContours;
	int numContours;
	int width;
	int height;
	int* relationTable;
	double** objColorFeatures;
	int nbins_L;
	int nbins_a;
	int nbins_b;

public:
	std::vector<std::string> objNames;
	std::vector<int> matchId; // Non-zero if a contour matche is found and the value is the object Index
	IplImage* imgc1c2c3_float;
	double* mXYZ;
	vector<ClosedContour*> objContours;	
	
	ROBOT();
	ROBOT(char* xmlFile);
	~ROBOT();
	
	//Reading members!
	int contourCount();
	int GetWidth(){ return width;}
	int GetHeight() { return height;}
	void readPtCloud(char*);
	void readXML(char* xmlFile);

	//For visualization
	int drawContour(int ind);
	int drawContour(IplImage* canvass,int ind);
	int drawContour(IplImage* canvass,int ind, CvScalar color);  
	int drawAllContours();
	int showImg();
	IplImage* findInsideOfContour(int ind);

	//Perception for interaction
	void spatialRelationBtwContours(); 

	//For Recognition
	void findSurfKeyPoints();
	void matchContour(int contourInd);
	void matchContours(); // match all contours 
	void loadKnowledge();
	void drawMatchingContour();
	void nameObjects();
	void convertbgrToc1c2c3();
	void readObjColorFeatures(char *, int nL, int na, int nb);
	void recognizeObjects();


	//For display purposes!
	void showObjectCenters();
	void showSurfaceNormals();
	int showResults();
	int showDepthBoundary();
	int showGradientMap();
	int showAllFixPts();
	int showImageSeq();
	int loadImageSeq();
	int showOccludingContours();
};
