//-----------------------------------------------
// Segmentation Layer
// Author: Ajay K Mishra
// Created on July 19, 2010
// Modified on Dec 01, 2010
//-----------------------------------------------

#ifndef _SEG_LAYER
#define _SEG_LAYER

#include <cv.h>
#include <highgui.h>
#include "savgol.h"
#include "sobelEdge.h"
#include "pbBoundary.h"
#include "misc.h"
#include "segFixatedRegion.h"

class segLayer{
 protected:
  IplImage* img;
  IplImage* flowU;
  IplImage* flowV;
  IplImage* dispMap;
  IplImage* edgeGrad;
  IplImage* edgeOri;
  IplImage* pbBoundary;
  IplImage* mask;
  CvMemStorage* storage;
  double*   groundColorHist_3D;

  int width;
  int height;
  int numOri;
  int nL, na, nb;

  bool flowFLAG;
  bool surfaceFLAG;
  
  //for weighted segmentation
  int* fixX;
  int* fixY;
  double* fixWeight;
  int fixNum;
  double cenX;
  double cenY;

  //particles parameters
  double sigma[7];
  cv::Mat Sk;
  
  double Delta;
  
  double N_threshold;
  cv::Mat Rk;
  cv::Mat R1;
  
  cv::Mat Ck;
  cv::Mat C1;
  
  cv::Mat A;
  
  
  double* yq;
  double* eq;
  double* ty;
  double* vect_edge1;
  double* vect_edge2;
  double* vect_edge3;
  double* yk;
  double* ek;
  double* py;
  double* rho;
  
  int histC;
  
  double* imgData;
  

    
  
  std::vector<CvPoint> fixPts;
  CvPoint currFixPt;
  
  //std::vector<CvSeq *> allContours;

  public:
  //constuctor & destructors
  segLayer();
  ~segLayer();
	
  bool disparityFLAG;
  std::vector<CvSeq *> allContours;
  
  // I/O
  int readImage(char* imgFileName);
  int setImage(IplImage* imgPtr);
  int getImgWidth();
  int getImgHeight();
  int copyToEdgeGrad(IplImage* tmp);
  int copyToEdgeOri(IplImage* tmp);
  int copyEdgeGrad(IplImage* tmp);
  int copyEdgeOri(IplImage* tmp);
  int copyToMask(IplImage* tmp);
  // TCL
  IplImage* getCurrSegImg();
  IplImage* getCurrPb();
  IplImage* getCurrMask();
IplImage* getCurrMasknSize(double& size);
  
  //Cues
  int edgeCGTG();
  int edgeBG();
  int edgeSobel();
  int setDisparity(IplImage* );
  int readDisparity(char*);
  int readFlow(char* txtFileName);
  int readFlow_flo(char* floFileName);
  int setU(IplImage* u);
  int setV(IplImage* v);
  int learnTableSurface(IplImage *tableImg, int nL, int na, int nb);
  int learnTableSurface(char *tableImgName, int nL, int na, int nb);

  //pb
  int generatePbBoundary();
  int generatePbBoundary(IplImage* mask);

  //fixation
  int selectFixPt_interactive();
  int selectFixPt_auto();
  int readFixPts(char* fixTxtFile);
  int assignFixPt(int x, int y);
  int getNumFixPts();
  int getFixPt(CvPoint&, int ind);
  CvPoint getFixPt(int ind);
  
  //Memory
  int allocateMemForContours();	
  int deallocateMemForContours();

  //Segmentation
  int segmentCurrFix();
  int segmentSpecificFix(int ind);
  int segmentAllFixs();
  int generateRegionMask(IplImage* mask, int contourInd);  
  int clearAllContours();
  
  //Save outputs
  int saveRegions(char* prefixTag);
  int saveEdgeMap(char* prefixTag);
  int savePbBoundary(char* prefixTag);
//TCL
  int saveRegionsnRGB(char* prefixTag);
    
  //Display
  int displayImg(int delay);
  int displayMask(int delay);
  int displayEdge(int delay);
  int displayPbBoundary(int delay);
  int displayRegion(int ind);
  int displayFlowMag(int delay);
  int displayCurrSegs(int delay);
  int displayCurrFixPt(int delay);
  int displayAllFixPts(int delay);
  int saveFixPtsImage(char* prefixTag);
  
  //particle filter
  int bin(double zi , double *vect_edge , int Nbin);
  void randini(void);
  void pdfcolor_ellipserand(double* Z, double* y, double* e, int N, double* py,int Npdf, double* vect_edge1, double* vect_edge2, double* vect_edge3, int width, int height);
  void fixationToParticles(int);
  void particle_resampling(double *pdf , double *indice , int N);
  void pdfcolor_mask(double* Z, int *Mask, double* py, double* vect_edge1, double* vect_edge2, double* vect_edge3, double width, double height);
  void updateTargetDis();
  
  double* cholesky(double *A, int n);
};


#endif
