//segmentation Layer 
//Author: Ajay K Mishra
//Date: July 19, 2010
//Modified on: Dec 01, 2010
//-------------------------------------------------
#include "segmentationLayer.h"

#include <math.h>
#include <time.h>
#include <iostream>
#include <fstream>

using namespace std;


#define NUMERICS_FLOAT_MIN 1.0E-37
#define PI 3.14159265358979323846
#define NOBIN -1

#ifndef max
    #define max(a,b) (a >= b ? a : b)
    #define min(a,b) (a <= b ? a : b)
#endif


#define mix(a , b , c) \
{ \
	a -= b; a -= c; a ^= (c>>13); \
	b -= c; b -= a; b ^= (a<<8); \
	c -= a; c -= b; c ^= (b>>13); \
	a -= b; a -= c; a ^= (c>>12);  \
	b -= c; b -= a; b ^= (a<<16); \
	c -= a; c -= b; c ^= (b>>5); \
	a -= b; a -= c; a ^= (c>>3);  \
	b -= c; b -= a; b ^= (a<<10); \
	c -= a; c -= b; c ^= (b>>15); \
}

#define znew   (z = 36969*(z&65535) + (z>>16) )
#define wnew   (w = 18000*(w&65535) + (w>>16) )
#define MWC    ((znew<<16) + wnew )
#define SHR3   ( jsr ^= (jsr<<17), jsr ^= (jsr>>13), jsr ^= (jsr<<5) )
#define randint SHR3
#define rand() (0.5 + (signed)randint*2.328306e-10)

#ifdef __x86_64__
    typedef int UL;
#else
    typedef unsigned long UL;
#endif

static UL jsrseed = 31340134 , jsr;

//--------------------------**
//Constructor
segLayer::segLayer(){
  flowFLAG = false;
  surfaceFLAG = false; 
  disparityFLAG = false;
  img = NULL;
  flowU = NULL;
  flowV = NULL;
  edgeGrad = NULL;
  edgeOri = NULL;
  pbBoundary = NULL;
  groundColorHist_3D = NULL;
  numOri = 8;
  //printf("1\n");
  
  Delta = 10;
  
  
  fixNum = 500;
  fixX = (int*)malloc(sizeof(int)*fixNum);
  fixY = (int*)malloc(sizeof(int)*fixNum);
  fixWeight = (double*)malloc(sizeof(double)*fixNum);
  
  N_threshold       = 6.*fixNum/10;
  
  for(int i=1;i<fixNum;i++)
    *(fixWeight+i) = 1.0/fixNum;
    
  //printf("2\n");
  
  sigma = {30.0,1.0,30.0,1.0,4.0,4.0,0.14}; // sigma[7] = 8*(pi/180)
  //printf("2.1\n");
  R1 = cv::Mat::zeros(7, 7, CV_64FC1);
  //printf("2.2\n");
  for(int i=0;i<7;i++)
    R1.at<double>(i,i) = sigma[i];
  cout << "R1 = "<< endl << " "  << R1 << endl << endl;
  //printf("3\n");
  
  Sk = cv::Mat::zeros(7,fixNum,CV_64FC1);
  //state covariance
  double sigmay = sqrt(0.35);
  double sigmaHx = 0.1;
  double sigmaHy = 0.1;
  double sigmatheta = 0.0524; //3*pi/180;
  
  double rk[] = {sigmay*sigmay*Delta*Delta*Delta/3.0,sigmay*sigmay*Delta*Delta/2.0,0,0,0, 0, 0,
                 sigmay*sigmay*Delta*Delta/2.0,sigmay*sigmay*Delta,0,0,0,0,0,
                 0,0,sigmay*sigmay*Delta*Delta*Delta/3.0,sigmay*sigmay*Delta*Delta/2.0,0,0,0,
                 0,0,sigmay*sigmay*Delta*Delta/2.0,sigmay*sigmay*Delta,0,0,0,
                 0,0,0,0,sigmaHx*sigmaHx,0,0,
                 0,0,0,0,0,sigmaHy*sigmaHy,0,
                 0,0,0,0,0,0,sigmatheta*sigmatheta};
                 
  double *crk = cholesky(rk,7);               
  
      
  //cholesky decomposition               
  Ck = cv::Mat( 7, 7, CV_64FC1, crk ).clone();
  
  C1 = R1.clone();
  
  cout << "Rk = " << Rk << endl;  
                 
  double a[] = {1,Delta,0,0,0,0,0,
                0,1,0,0,0,0,0,
                0,0,1,Delta,0,0,0,
                0,0,0,1,0,0,0,
                0,0,0,0,1,0,0,
                0,0,0,0,0,1,0,
                0,0,0,0,0,0,1};
  //double *ca = cholesky(a,7);
                
  A = cv::Mat(7,7,CV_64FC1,a).clone();                
  
  
  
  
  //cv::Mat D = cv::Mat::ones(7,7,CV_64FC1);
  
  //cv::Mat U(7,7,CV_64FC1);
  
  //cv::SVD svdK(Rk,cv::SVD::FULL_UV);
  //cv::SVD svd1(R1,cv::SVD::FULL_UV);
    
  //cv::Mat wTmp = cv::Mat::zeros(7,7,CV_64FC1); 
  
  //cout << svdK.u << endl;
  //cout << svdK.w << endl;
  //cout << svdK.vt << endl;
  
  //for(int i=0;i<7;i++)
  //{
    //wTmp.at<double>(i,i) = svdK.w.at<double>(i);
  //}
   
  //cv::sqrt(wTmp, wTmp);
  //Ck = svdK.u * wTmp;
  
  //for(int i=0;i<7;i++)
  //{
    //wTmp.at<double>(i,i) = svd1.w.at<double>(i);
  //}
  //cv::sqrt(wTmp, wTmp);
  //C1 = svd1.u * wTmp;
  
  cout << Ck << endl;
  
  cout << C1 << endl;  

  //cvSVD(Rk, D, U, Ck, CV_SVD_U_T|CV_SVD_V_T); // Rk = Ck' I Ck;
  //cvSVD(R1, D, U, C1, CV_SVD_U_T|CV_SVD_V_T); // R1 = C1' I C1;
  
  histC = 8*8*8;
  
  yq = (double*)calloc(2,sizeof(double));
  eq = (double*)calloc(3,sizeof(double));
  
  vect_edge1 = (double*)calloc(9,sizeof(double));
  vect_edge2 = (double*)calloc(9,sizeof(double));
  vect_edge3 = (double*)calloc(9,sizeof(double));
  yk = (double*)calloc(2*fixNum,sizeof(double));
  ek = (double*)calloc(3*fixNum,sizeof(double));
  
  ty = (double*)calloc(histC,sizeof(double));
  py = (double*)calloc(histC*fixNum,sizeof(double));
  rho = (double*)calloc(fixNum,sizeof(double));
  
    for(int i = 0;i<9;i++)
    {
        *(vect_edge1+i) = i*32;
        *(vect_edge2+i) = i*32;
        *(vect_edge3+i) = i*32;
    }    
     
  
}

//---------------------------**
//Destructor
segLayer::~segLayer(){
  // BUG: openCV complains img is not the right type...
  //if(img!=NULL) cvReleaseImage(&img);
  if(edgeGrad!=NULL) cvReleaseImage(&edgeGrad);
  if(edgeOri!=NULL) cvReleaseImage(&edgeOri);
  if(flowU!=NULL) cvReleaseImage(&flowU);
  if(flowV!=NULL) cvReleaseImage(&flowV);
  if(surfaceFLAG) free(groundColorHist_3D);
  if(pbBoundary != NULL) cvReleaseImage(&pbBoundary);	
}

//chloskey decomposition

double* segLayer::cholesky(double *A, int n) {
    double *L = (double*)calloc(n * n, sizeof(double));
    if (L == NULL)
        exit(EXIT_FAILURE);
 
    for (int i = 0; i < n; i++)
        for (int j = 0; j < (i+1); j++) {
            double s = 0;
            for (int k = 0; k < j; k++)
                s += L[i * n + k] * L[j * n + k];
            L[i * n + j] = (i == j) ?
                           sqrt(A[i * n + i] - s) :
                           (1.0 / L[j * n + j] * (A[i * n + j] - s));
        }
 
    return L;
}

//
void segLayer::pdfcolor_ellipserand(double* Z, double* y, double* e, int N, double* py, int Npdf, double* vect_edge1, double* vect_edge2, double* vect_edge3, int width, int height)
{
    //double *Z , *y , *e  ;
	//double *py , *zi , *yi;
	//double *vect_edge1 , *vect_edge2 , *vect_edge3;
	double twoPI = 2*PI , u , angle , val ;
	double theta , cos_theta , sin_theta , tmp1 , tmp2 , s , t , invM;
	/*double NAN = mxGetNaN();*/
	//const int *dimsZ=NULL , *dimsy=NULL , *dimse=NULL , *dimsvect_edge1=NULL , *dimsvect_edge2=NULL , *dimsvect_edge3=NULL;
	//int *dimspy=NULL , *dimszi=NULL , *dimsyi=NULL;
	int i , j , n , m , nm , r , NNpdf , Nx , Ny , Nz , Nxy;
	int numdimsZ , numdimsy , numdimse , numdimsvect_edge1 , numdimsvect_edge2 , numdimsvect_edge3;
	int numdimspy , numdimszi , numdimsyi; 	
	int iNpdf2, iNpdf3 , iM , i2 , i3 , j2 , j3 , ndx , kx , ky , kz , M , j2iNpdf2 , j3iNpdf3;
	int floors , floort , compteur;
	
	n = height;
	m = width;
	nm = n*m;
    randini();
    //cout << width << height << endl;
    Nx = Ny = Nz = 8;
    
    M        = Nx*Ny*Nz;
	Nxy      = Nx*Ny;
	invM     = 1.0/(double)(M);
	
    NNpdf = Npdf*N;
    
	for (i = 0 ; i < N ; i++)
	{
		iNpdf2     = i*Npdf*2;
		iNpdf3     = i*Npdf*3;
		iM         = i*M;
		i2         = i*2;
		i3         = i*3;
		theta      = e[2 + i3];
		cos_theta  = cos(theta);
		sin_theta  = sin(theta);
		compteur   = 0;
		//printf("2.1\n");
		for (j = 0 ; j < Npdf ; j++)
		{	
			j2                  = 2*j;
			j2iNpdf2            = j2 + iNpdf2;
    	    j3                  = 3*j;
			j3iNpdf3            = j3 + iNpdf3;
			u                   = rand();
			angle               = twoPI*rand();
			tmp1                = (e[0 + i3]*u)*cos(angle);
			tmp2                = (e[1 + i3]*u)*sin(angle);
			s                   = y[0 + i2] + (cos_theta*tmp1 - sin_theta*tmp2);
			t                   = y[1 + i2] + (sin_theta*tmp1 + cos_theta*tmp2);
			//yi[0 + j2iNpdf2]    =  s;
			//yi[1 + j2iNpdf2]    =  t;
            //printf("2.2\n");
			if ((s < 1) || (s > m) || (t < 1) || (t > n))
			{
				//zi[0 + j3iNpdf3]    = 0;
				//zi[1 + j3iNpdf3]    = 0;
				//zi[2 + j3iNpdf3]    = 0;
				kx                  = NOBIN;
				ky                  = NOBIN;
				kz                  = NOBIN;
			}
			else
			{
                floors              = floor(s);
				floort              = floor(t);
				ndx                 = floort + (floors - 1)*n - 1;
				s                   = (s - floors);
				t                   = (t - floort);
				r                   = 0*nm;
				
				//cout << Z[ndx] << endl;
				//printf("2.3\n");
				val                 = (Z[ndx + r]*(1 - t) + Z[ndx + r + 1]*t)*(1 - s) + ( Z[ndx + r + n]*(1 - t) + Z[ndx + n + 1 + r]*t )*s;
				//zi[0 + j3iNpdf3]    = val;
				//printf("2.4\n");
                kx                  =  bin(val , vect_edge1 , Nx);
				//cout<<val<<' '<<kx<<endl;
				//printf("2.5\n");
				r                   = 1*nm;
				val                 = (Z[ndx + r]*(1 - t) + Z[ndx + r + 1]*t)*(1 - s) + ( Z[ndx + r + n]*(1 - t) + Z[ndx + n + 1 + r]*t )*s;
				//zi[1 + j3iNpdf3]    = val;
				//printf("2.6\n");
				ky                  =  bin(val , vect_edge2 , Ny);
				//cout<<ky<<endl;
				//printf("2.7\n");
				r                   = 2*nm;
				val                 = (Z[ndx + r]*(1 - t) + Z[ndx + r + 1]*t)*(1 - s) + ( Z[ndx + r + n]*(1 - t) + Z[ndx + n + 1 + r]*t )*s;
				//zi[2 + j3iNpdf3]    = val;
				//printf("2.8\n");
				kz                  =  bin(val , vect_edge3 , Nz);
				//cout<<kz<<endl;
				//printf("2.9\n");
			}
			if ((kx != NOBIN) && (ky != NOBIN) && (kz != NOBIN))
			{
				py[kx + ky*Nx + kz*Nxy + iM]++;
				compteur++;
			}
		}
		if (compteur != 0)
		{
			invM        = 1.0/(double)compteur;
		}
		else
		{
			invM = 1.0;
		}
		for (j = 0 ; j < M ; j++)	
		{
		
		    //cout<<invM<<endl;
			py[j + iM] = (py[j + iM])*invM;
		}
	}
}

void segLayer::pdfcolor_mask(double* Z, int *Mask, double* py, double* vect_edge1, double* vect_edge2, double* vect_edge3, double width, double height)
{
	
	int kx,ky,kz,Nx, Ny, Nz, M, Nxy;
    double val1,val2,val3;
    int compteur = 0;
    double invM;
    Nx = Ny = Nz = 8;
    int n,m;
    n = height;
	m = width;
    M        = Nx*Ny*Nz;
	Nxy      = Nx*Ny;
	
	//printf("4.01\n");
	for(int i = 0; i< m; i++)
	{
	    for(int j=0;j<n;j++)
	    {
	        val1 = Z[i*n + j];
	        //printf("4.000\n");
	        val2 = Z[n*m + i*n + j];
	        val3 = Z[2*n*m + i*n + j];
	        //rintf("4.001\n");
	        kx = bin(val1,vect_edge1,Nx);
	        ky = bin(val2,vect_edge2,Ny);
	        kz = bin(val3,vect_edge3,Nz);
	        
	        //printf("4.002\n");
	        if(Mask[i*n + j] == 1)
	        {
	            py[kx + ky*Nx + kz*Nxy]++;
			    compteur++;
		    }
	    }	    
	}
	//printf("4.02\n");
	if (compteur != 0)
	{
		invM        = 1.0/(double)compteur;
	}
	else
	{
		invM = 1.0;
	}
	
	for (int i = 0; i < M; i++)
	{
		py[i] = (py[i])*invM;   
	}
    
}

/*------------------------------------------------------------*/
int segLayer::bin(double zi , double *vect_edge , int Nbin)
{
	int k  = NOBIN;
	int k0 = 0 , N1 = Nbin , k1 = N1;
	
	if ((zi >= vect_edge[0]) && (zi <= vect_edge[N1]))
	{
		k = (k0 + k1)/2;
		while (k0 < k1 - 1)
		{
			if (zi >= vect_edge[k]) 
			{
				k0 = k;
			}
			else 
			{
				k1 = k;
			}
			k = (k0 + k1)/2;
		}
		k = k0;
	}
	return k;
}
/* ----------------------------------------------------------------------- */
void segLayer::randini(void) 
{	
	/* SHR3 Seed initialization */
	jsrseed  = (UL) time( NULL );
	jsr     ^= jsrseed;
}
/* ----------------------------------------------------------------------- */

void segLayer::fixationToParticles(int loopCt)
{
    
    
    //ty = (double*)calloc(histC,sizeof(double));
    py = (double*)calloc(histC*fixNum,sizeof(double));
    cv::Mat NorRandMat(7,fixNum,CV_64FC1);
    
    
    
	IplImage* img_hsv = cvCreateImage(cvSize(img->width, img->height), IPL_DEPTH_8U, 3);
	cvCvtColor(img,img_hsv,CV_RGB2HSV);
    printf("1.01\n");
    cout << img->width << ' ' << img->height << endl;
    //mapping opencv image format to matlab format!!! IMPORTANT!!!
    for (int i = 0; i < img->height; i++)
    {
        for(int j = 0; j < img->width; j++)
        {
            imgData[j*img->height + i] = (double)CV_IMAGE_ELEM(img_hsv, uchar, i, 3*j);
            imgData[img->width*img->height + j*img->height + i] = (double)CV_IMAGE_ELEM(img_hsv, uchar, i, 3*j + 1);
            imgData[2*img->width*img->height + j*img->height + i] = (double)CV_IMAGE_ELEM(img_hsv, uchar, i, 3*j + 2);
        }
        
    }
    printf("1.02\n");
//    ofstream myfile;
//    myfile.open ("/home/erratic/tmp_data.txt");
//    for (int i = 0; i < img->height; i++)
//    {
//        for(int j = 0; j < img->width; j++)
//        {
//            myfile << imgData[img->width*img->height + j*img->height + i] << ' ';
//        }
//        myfile << endl;
//    }
//    myfile.close();
//    for(int y=0; y<height; y++)
//	for(int x=0; x<width; x++){
//		*(pb_ptr + width*y + x)  = CV_IMAGE_ELEM(pbBoundary, float, y, x);
//		*(ori_ptr + width*y + x) = CV_IMAGE_ELEM(edgeOri, float, y, x);
//		*(fg_ptr + width*y + x)  = 0;
//	}
    //cv::Mat NorRandMat(7,fixNum,CV_64FC1);
    if(loopCt == 0)
    {
        *yq = currFixPt.x + 1; //x is the height 480
        *(yq+1) = currFixPt.y + 1; //y is the width 640

    
        
        *eq = 10.0;
        *(eq+1) = 10.0;
        *(eq+2) = 0.0;
        //target distribution
        pdfcolor_ellipserand(imgData, yq, eq, 1, ty, 1000, vect_edge1, vect_edge2, vect_edge3, img->width, img->height);
        
        for(int i = 0; i<fixNum; i++)
        {
        
            Sk.at<double>(0,i) = yq[0]+1; //matlab index, will be adjusted in ellipserand
            Sk.at<double>(1,i) = 0.0;
            Sk.at<double>(2,i) = yq[1]+1;
            Sk.at<double>(3,i) = 0.0;
            Sk.at<double>(4,i) = eq[0];
            Sk.at<double>(5,i) = eq[1];
            Sk.at<double>(6,i) = eq[2];
        }
    
        
     
        cv::randn(NorRandMat, 0,1);
    
    //Sk = Sk + C1.t() * NorRandMat;
    //particle initialization
        Sk = Sk + C1 * NorRandMat;
    }

    
    
    //first iterative
    cv::randn(NorRandMat, 0,1);
    //cout << "NorRandMat = " << NorRandMat << endl;
    Sk = A*Sk + Ck*NorRandMat;
    //cout << "C1 = "<< endl << " "  << C1 << endl << endl;
    //cout << "Sk = "<< endl << " "  << Sk << endl << endl;
    
    for(int i= 0;i<fixNum;i++)
    {
    
    //switch here?
        *(yk+2*i) = Sk.at<double>(0,i);
        *(yk+2*i+1) = Sk.at<double>(2,i);
        *(ek+3*i) = Sk.at<double>(4,i);
        *(ek+3*i+1) = Sk.at<double>(5,i);
        *(ek+3*i+2) = Sk.at<double>(6,i);
    }
//    for(int i = 0; i<fixNum; i++)
//    {
//        cout << "yk = "<< endl << " "  << yk[2*i] << " " << yk[2*i+1] << endl;
//    }
    pdfcolor_ellipserand(imgData, yk, ek, fixNum, py, 1000, vect_edge1, vect_edge2, vect_edge3, img->width, img->height);
    
//    for(int i = 0; i<fixNum; i++)
//    {
//        for(int j = 0; j<histC; j++)
//        {
//            cout << py[i*fixNum+j] << ' ';
//        }
//        cout << endl << endl;
//    }
    double sigma_color       = 0.2;      // Measurement Color noise
    double cte1_color        = 1/(2*sigma_color*sigma_color);
    double cte2_color        = (1/(sqrt(2*PI)*sigma_color));
    double weiSum = 0.0;
    
    for(int i = 0; i<fixNum; i++)
    {
        double tmpSum = 0.0;
        for(int j = 0; j<histC; j++)
        {
            tmpSum += sqrt(py[i*histC+j] * ty[j]);
            //tmpSum += py[i*fixNum+j] * log(py[i*fixNum+j]/ty[j]);
        }
        rho[i] = cte2_color*exp((tmpSum-1)*cte1_color);
        fixWeight[i] = fixWeight[i] * rho[i];
        weiSum += fixWeight[i];
        //cout << rho[i] << ' ';
    }
    cout<<endl;
    int locX, locY;
    
    cout << "particle weights:" << endl;
    
    double isWeiSum = 0.0;
    for(int i = 0; i<fixNum; i++)
    {
        fixWeight[i] = fixWeight[i] / weiSum;  
        isWeiSum += fixWeight[i];
             
        cout << fixWeight[i] << ' ';
    }
    cout << endl << isWeiSum << endl;
    // if resampling ?
    double N_eff = 0.0;
    
    for(int i = 0 ; i < fixNum; i++)
    {
        N_eff += fixWeight[i]*fixWeight[i];
    }
    
    N_eff = 1/N_eff;
    cout << "resample "<< N_eff  << " " << N_threshold << endl;
    if(N_eff < N_threshold)
    {
        
        double* ind = (double *)malloc(sizeof(double)*fixNum);
        particle_resampling(fixWeight, ind, fixNum);
        
        cv::Mat cSk = Sk.clone();
        double* cFixWeight = (double *)malloc(sizeof(double)*fixNum);
        
        memcpy (cFixWeight,fixWeight,sizeof(double)*fixNum);
        
        for(int i=0;i<fixNum;i++)
        {
            cout<<ind[i]<<" ";
            for (int j=0;j<7;j++)
            {
                Sk.at<double>(j,i) = cSk.at<double>(j,(int)ind[i]-1); 
            }
            fixWeight[i] = 1.0/fixNum;
            //cout << fixWeight[i] << ' '; 
        }
        //cout<<endl;
        
    } 
    
    for(int i = 0; i<fixNum; i++)
    {
        //*(fixWeight+i) = *(fixWeight+i) / weiSum;
        locX = floor(Sk.at<double>(0,i));
        locY = floor(Sk.at<double>(2,i));
        //cout << "LocX:" << locX << "LocY:"<<locY<< endl;
        if (locX < 0)
            *(fixX + i) = 0;
        else if(locX >= img->width)
            *(fixX + i) = img->width - 1;
        else
            *(fixX + i) = locX;
        
        if(locY < 0)
            *(fixY + i) = 0;
        else if(locY >= img->height)
            *(fixY + i) = img->height - 1;
        else
            *(fixY + i) = locY;
            
        
    }
    
    //weighted center
    weiSum = 0.0;
    cenX = 0.0;
    cenY = 0.0;
    for(int i = 0; i< fixNum; i++)
    {
        cenX += fixX[i] * fixWeight[i];
        cenY += fixY[i] * fixWeight[i];
        weiSum += fixWeight[i];
    }    
    
    cenX = cenX/weiSum;
    cenY = cenY/weiSum;
    
    //
    
    IplImage* tmpImg = cvCloneImage(img);
    
    double* weiColor = (double*)malloc(sizeof(double)*fixNum);
    double weiMin = 10000;
    double weiMax = 0;
    for(int i=0;i<fixNum;i++)
    {
        if(fixWeight[i] < weiMin)
            weiMin = fixWeight[i];
        if(fixWeight[i] > weiMax)
            weiMax = fixWeight[i];
    }
    
    for(int i=0; i<fixNum; i++)
    {
        if(weiMin == weiMax)
            weiColor[i] = 125;
        else
            weiColor[i] = 255 * (fixWeight[i] - weiMin)/(weiMax - weiMin);
    }
    for(int i=0;i<fixNum;i++)
    {
         // default construction of an object of type CvPoint
        CvPoint P_1 = cvPoint(*(fixX + i),*(fixY + i));

  	    cvCircle(tmpImg, P_1, 3, CV_RGB(0, (int)weiColor[i], 0), -1);
  	    
  	}
	cvShowImage("Fixation Distribution",tmpImg);
	cvReleaseImage(&tmpImg);
    //delete imgData;
    //delete ty;
    delete py;
    

}

void segLayer::updateTargetDis()
{
    int* Mask = (int*)malloc(sizeof(int)*img->height*img->width);
    
    printf("3.01\n");
    for (int i = 0; i < img->height; i++)
    {
        for(int j = 0; j < img->width; j++)
        {
            if(CV_IMAGE_ELEM(mask, uchar, i, j) == 255) 
                Mask[j*img->height + i] = 1;
            else
                Mask[j*img->height + i] = 0;
            //imgData[img->width*img->height + j*img->height + i] = (double)CV_IMAGE_ELEM(img_hsv, uchar, i, 3*j + 1);
            //imgData[2*img->width*img->height + j*img->height + i] = (double)CV_IMAGE_ELEM(img_hsv, uchar, i, 3*j + 2);
        }
        
    }
    printf("3.02\n");
    //cout<<imgData[0]<<endl;
    pdfcolor_mask(imgData, Mask, ty, vect_edge1, vect_edge2, vect_edge3, img->width, img->height);
    printf("3.03\n");
    
}


void segLayer::particle_resampling(double *pdf , double *indice , int N)	 
{
	int    i  , j;
	double sumcdf , sumuu , one = 1.0 , tmp;
	double *cdf , *uu;

	cdf       = (double *)malloc(N*sizeof(double)); 
	uu        = (double *)malloc(N*sizeof(double));

    randini();
	/* Compute Scaled empirical CDF from PDF & Sorted Uniform samples*/

	cdf[0]    = pdf[0];
	uu[0]     = -log(rand());
	sumcdf    = cdf[0];
	sumuu     = uu[0];

	for(i = 1 ; i < N ; i++)
	{
		tmp       = pdf[i] + cdf[i - 1];
		cdf[i]    = tmp;
		tmp       = uu[i - 1] - log(rand());
		uu[i]     = tmp;
		sumcdf   += cdf[i];
		sumuu    += uu[i];		 
	}
	sumcdf = one/sumcdf;
	sumuu  = one/sumuu;

	/* Normalize */

	for (i = 0 ; i < N ; i++)
	{
		cdf[i]   *= sumcdf;
		uu[i]    *= sumuu;
	}

	/*  On recopie les N premiers éléments de u_ord dans le vecteur uu. Le N + 1 élément est égal à 2. */

	i = 0;
	j = 0;
	while (i < N)
	{
		if ( (uu[i] < cdf[j]) | (j == N - 1))
		{
			indice[i] = j + 1;
			i++;
		}
		else
		{
			if (j < N)
			{
				j++;
			}
		}
	}	
	free(uu);
	free(cdf);
}


//---------------------------**
//Read images
int segLayer::readImage(char *imgFileName){
  IplImage* tmp = cvLoadImage(imgFileName,-1);
  if (tmp == NULL){
    fprintf(stderr, "\n unable to read %s",imgFileName);
    exit(1);
  }
  if (tmp->nChannels != 3){
    fprintf(stderr, "\nerror: program accepts color images only");
    exit(1);
  }

  img    = cvCloneImage(tmp);
  width  = img->width;
  height = img->height;

  cvReleaseImage(&tmp);
  return 0;
}

int segLayer::setImage(IplImage *tmp){
  if (tmp == NULL){
    fprintf(stderr, "\n image is null");
    exit(1);
  }
  if (tmp->nChannels != 3){
    fprintf(stderr, "\nerror: program accepts color images only");
    exit(1);
  }
  img    = tmp;
  width  = img->width;
  height = img->height;
  
  imgData = (double*)malloc(sizeof(double)*img->height*img->width*3);
  return 0;
}

int segLayer::getImgWidth(){
	return width;
}
int segLayer::getImgHeight(){
	return height;
}
int segLayer::copyToEdgeGrad(IplImage* tmp){   
	if (tmp == NULL ){
		return 1;
	}
	else if( tmp->width != width || tmp->height != height){
		return 1;
	}
	if (edgeGrad == NULL ){
		edgeGrad = cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, 1);
	}
	cvCopy(tmp, edgeGrad);
	return 0;
}

int segLayer::copyToEdgeOri(IplImage* tmp){
	if (tmp == NULL ){
		return 1;
	}
	else if( tmp->width != width || tmp->height != height){
		return 1;
	}
	if (edgeOri == NULL){
		edgeOri = cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, 1);
	}
	cvCopy(tmp, edgeOri);
	return 0;
}

int segLayer::copyEdgeGrad(IplImage* tmp){
	if (tmp == NULL ){
		return 1;
	}
	else if( tmp->width != width || tmp->height != height){
		return 1;
	}
	else{
		cvCopy(edgeGrad, tmp);
		return 0;
	}
}

int segLayer::copyEdgeOri(IplImage* tmp){
	if (tmp == NULL ){
		return 1;
	}
	else if( tmp->width != width || tmp->height != height){
		return 1;
	}
	else{
		cvCopy(edgeOri, tmp);
		return 0;
	}
}

int segLayer::copyToMask(IplImage* tmp){
	if (tmp == NULL){
		return 1;
	}
	else if( tmp->width != width || tmp->height != height || tmp->depth != IPL_DEPTH_8U){
		printf("\n in setMask: either the mask is of wrong size or it is not uchar \n");
		return 1;
	}
	else{
		mask = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
		if(tmp->nChannels == 3){
			cvCvtColor(tmp, mask, CV_BGR2GRAY);
			cvThreshold(mask, mask, 100, 255,CV_THRESH_BINARY);
		}
		else{
			cvThreshold(tmp, mask,  100, 255,CV_THRESH_BINARY);		
		}		
		return 0;
	}
}

//TCL: get the current segmented image
IplImage* segLayer::getCurrSegImg(){
	if(img != NULL) {
		IplImage* tmp      = cvCloneImage(img);
	  	int len = allContours.size();
	  	cvDrawContours(
			 tmp,
			 allContours[len-1],
			 CV_RGB(0,255,0),
			 CV_RGB(255,255,255),
			 -1,
			  5);
		return tmp;
	}
	else
	 return NULL;

}

//TCL: get the current pb image
IplImage* segLayer::getCurrPb(){
	if(pbBoundary != NULL) {
		IplImage* tmpImg = cvCreateImage(cvGetSize(pbBoundary), IPL_DEPTH_8U, 1);
  		cvScale(pbBoundary, tmpImg, 255);	
		return tmpImg;
	}
	else
	 return NULL;

}

//TCL: get current object Mask
IplImage* segLayer::getCurrMasknSize(double &size){
	IplImage* tmp = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
    int numContours = allContours.size();
	for(int i=0; i<numContours; i++){
		cvSetZero(tmp);
		printf("segmentation completed 2\n");
		cvDrawContours(
		 			tmp,
		 			//allContours[numContours-1],
					allContours[i],
		 			CV_RGB(255,255,255),
			 		CV_RGB(255,255,255),
		 			-1,
		  			-1);

		//quick test
		//cvNamedWindow("mask");
		//cvShowImage("mask", tmp);
		//cvWaitKey(3);

		//compute size
		size = cvContourArea(allContours[i], CV_WHOLE_SEQ );


	}
	
	//cvReleaseImage(&tmp);
	return tmp;

}

IplImage* segLayer::getCurrMask(){
	mask = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
    int numContours = allContours.size();
    cvSetZero(mask);
	for(int i=0; i<numContours; i++){
		
		printf("segmentation completed 2\n");
		cvDrawContours(
		 			mask,
		 			//allContours[numContours-1],
					allContours[i],
		 			CV_RGB(255,255,255),
			 		CV_RGB(255,255,255),
		 			-1,
		  			-1);

		//quick test
		//cvNamedWindow("mask");
		//cvShowImage("mask", tmp);
		//cvWaitKey(3);
	}
	
	//cvReleaseImage(&tmp);
	return mask;
	
}

//----------------------------------**
//Cues: color, intensity and texture
int segLayer::edgeCGTG(){
  if(edgeGrad == NULL){
    edgeGrad = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);    
  }
  if (edgeOri == NULL){
    edgeOri = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);
  }
  IplImage* img_float = cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, 3);
   cvConvertScale(img, img_float, 1.0);
// DEBUG: show image - a problem here?
/* cvConvertScale(img, img_float, (1.0f/255.0f));
 printf("%d\n", img->depth);
 cvNamedWindow("Test");
 cvShowImage("Test",img_float);
 cvWaitKey(-1);
//  cvSaveImage("Input1.jpg", img);
*/
   

  pbCGTG(img_float, edgeGrad, edgeOri);
  cvReleaseImage(&img_float);
  return 0;
}
 

int segLayer::edgeBG(){
  if(edgeGrad == NULL){
    edgeGrad = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);    
  }
  if (edgeOri == NULL){
    edgeOri = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);
  } 
  IplImage* img_float = cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, 3);
  cvConvertScale(img, img_float, 1.0);
  pbBG(img_float, edgeGrad, edgeOri);
  cvReleaseImage(&img_float);
  return 0;
} 

int segLayer::edgeSobel(){
  if(edgeGrad == NULL){
    edgeGrad = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);    
  }
  if (edgeOri == NULL){
    edgeOri = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);
  }  
  IplImage* imgGray = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 1);
  cvCvtColor(img, imgGray, CV_BGR2GRAY);
  calcSobelEdge(imgGray, edgeGrad, edgeOri);
  //cv::Canny(imgGray, edgeGrad);
  //debug
  cvShowImage("Grad",edgeGrad);
  cvShowImage("Ori",edgeOri);
  
  return 0;
}


//---------------------------**
//Cues: optic flow
//1)
int segLayer::readFlow(char *flowTxtFile){
  	flowFLAG = true; 
  	flowU = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);
  	flowV = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);

  	FILE* fp_flow = fopen(flowTxtFile,"r");
  	if (fp_flow == NULL){
    		fprintf(stderr,"\n error: could not load %s ", flowTxtFile);
    		exit(1);
  	}	
  	char line[100];
  	for(int y=0; y<flowU->height; y++){
    		for(int x=0; x<flowU->width; x++){
      		if(fgets(line,90,fp_flow)!=NULL){
				float u,v;
				sscanf(line,"%f%f",&u,&v);
				CV_IMAGE_ELEM(flowU,float,y,x)=u;
				CV_IMAGE_ELEM(flowV,float,y,x)=v;
      		}
      		else{
				fprintf(stderr,"\n error: %s does not contain sufficient data",flowTxtFile);
				exit(1);
      		}
    		}
  	}
  	fclose(fp_flow);
  	return 0;
}

int segLayer::readDisparity(char* dispTxtFile){
	disparityFLAG = true;
	dispMap = cvCreateImage(cvSize(width,height), IPL_DEPTH_32F, 1);
	FILE* fp_disp = fopen(dispTxtFile, "r");
	if (fp_disp == NULL){
		fprintf(stderr,"\n error: could not load %s ", dispTxtFile);
		exit(1);
	}
	char line[100];	
	for(int y=0; y<dispMap->height; y++){
		for(int x=0; x<dispMap->width; x++){
			if(fgets(line,90,fp_disp) != NULL){
				float d;
				sscanf(line,"%f", &d);
				CV_IMAGE_ELEM(dispMap, float, y, x) = d;
			}
			else{
				fprintf(stderr,"\n error: %s does not contain sufficient data", dispTxtFile);
				exit(1);
			}
		}
	}	
}

//1.5)
int segLayer::readFlow_flo(char *filename){
  flowFLAG = true; 
  float TAG_FLOAT = 202021.25;
  flowFLAG = true; 
  if (flowU != NULL)
  	cvReleaseImage(&flowU);
  if (flowV != NULL)
  	cvReleaseImage(&flowV);

  flowU = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);
  flowV = cvCreateImage(cvGetSize(img), IPL_DEPTH_32F,1);
  if (filename == NULL)
    fprintf(stderr,"ReadFlowFile: empty filename");
  char *dot = strrchr(filename, '.');
  if (strcmp(dot, ".flo") != 0)
    fprintf(stderr, "ReadFlowFile (%s): extension .flo expected", filename);
  FILE *stream = fopen(filename, "rb");
  if (stream == 0)
    fprintf(stderr,"ReadFlowFile: could not open %s", filename);
  int width, height;
  float tag;    
  if ((int)fread(&tag,    sizeof(float), 1, stream) != 1 ||
      (int)fread(&width,  sizeof(int),   1, stream) != 1 ||
      (int)fread(&height, sizeof(int),   1, stream) != 1)
    fprintf(stderr, "ReadFlowFile: problem reading file %s", filename);  
  if (tag != TAG_FLOAT) // simple test for correct endian-ness
    fprintf(stderr,"ReadFlowFile(%s): wrong tag (possibly due to big-endian machine?)", filename);  
  // another sanity check to see that integers were read correctly (99999 should do the trick...)
  if (width < 1 || width > 99999)
    fprintf(stderr,"ReadFlowFile(%s): illegal width %d", filename, width);
  if (height < 1 || height > 99999)
    fprintf(stderr, "ReadFlowFile(%s): illegal height %d", filename, height);
  if (height != flowU->height || width != flowU->width)
    fprintf(stderr, "image size in %s does not match the frame size", filename);
 for (int y = 0; y < height; y++){
    for(int x= 0; x < width ; x++){
      float* ptr_u = (float*)((flowU)->imageData + y*((flowU)->widthStep)) + x;
      float* ptr_v = (float*)((flowV)->imageData + y*((flowV)->widthStep)) + x;
      if ((int)fread(ptr_u, sizeof(float), 1, stream) != 1)
	fprintf(stderr,"y=%d, ReadFlowFile(%s): file is too short\n", y, filename);  
      if ((int)fread(ptr_v, sizeof(float), 1, stream) != 1)
	fprintf(stderr,"ReadFlowFile(%s): file is too short", filename);
    }
  }  
  if (fgetc(stream) != EOF)
    fprintf(stderr,"ReadFlowFile(%s): file is too long", filename);
  fclose(stream);	
  return 0;
}

int segLayer::setDisparity(IplImage* inputDispMap){
	if (	inputDispMap->nChannels == 1 && 
		inputDispMap->depth == IPL_DEPTH_32F && 
		inputDispMap->width == width && 
		inputDispMap->height==height){
		
		disparityFLAG = true;
		dispMap = cvCloneImage(inputDispMap);
		//BUG: let the calling function release it...
		//cvReleaseImage(&inputDispMap);
		return 0;
	}	
	else{
		return 1;
	}
}

//2)
int segLayer::setU(IplImage* u){
  flowFLAG = true;

  if (u == NULL){
    fprintf(stderr,"\nNo optic flow \"u\" present");
    exit(1);
  }
  if (u->width != width || u->height != height){
    fprintf(stderr,"\nDimension mismatch between flow and rgb image");
    exit(1);
  }
  flowU = u;
  return 0;
}

//3)
int segLayer::setV(IplImage* v){
  flowFLAG = true;
  if (v == NULL){
    fprintf(stderr,"\nNo optic flow \"v\" present");
    exit(1);
  }
  if (v->width != width || v->height != height){
    fprintf(stderr,"\nDimension mismatch between flow and rgb image");
    exit(1);
  }
  flowV = v;
  return 0;
}


//-----------------------------**
//Cues: table distribution
//1)
int segLayer::learnTableSurface(IplImage *tableImg, int nL, int na, int nb){
  if(tableImg->depth != IPL_DEPTH_8U){
    fprintf(stderr, "error: table image should be uint8");
    exit(1);
  }
  this->nL = nL;
  this->na = na;
  this->nb = nb;
  if (groundColorHist_3D != NULL){
    delete [] groundColorHist_3D;
  }

  groundColorHist_3D = new double[nL*na*nb];
  getColorHist(&tableImg, 1, groundColorHist_3D, nL, na, nb);   
  surfaceFLAG = true;
  return 0;
}

int segLayer::learnTableSurface(char* tableImgName, int nL, int na, int nb){
  IplImage* tableImg = cvLoadImage(tableImgName, -1);
  if(tableImg->depth != IPL_DEPTH_8U || tableImg->nChannels !=3){
    fprintf(stderr, "error: table image should be uint8 and 3-channels");
    exit(1);
  }
  this->nL = nL;
  this->na = na;
  this->nb = nb;
  groundColorHist_3D = (double*)calloc(nL*na*nb, sizeof(double));
  
  IplImage* tableImg_c1c2c3 = cvCreateImage(cvGetSize(tableImg), IPL_DEPTH_32F, 3);
  IplImage* tableImg_float  = cvCreateImage(cvGetSize(tableImg), IPL_DEPTH_32F, 3);
  cvScale(tableImg, tableImg_float);
  bgrToc1c2c3(tableImg_float, tableImg_c1c2c3);
  getColorHist(&tableImg_c1c2c3, 1, groundColorHist_3D, nL, na, nb);   

  surfaceFLAG = true;
  cvReleaseImage(&tableImg);
  cvReleaseImage(&tableImg_float);
  cvReleaseImage(&tableImg_c1c2c3);
  return 0;
}



//-----------------------------**
//pb: probabilistic boundary edge map
//1)
int segLayer::generatePbBoundary(){
  	IplImage* pbBoundaryFlow; 
  	IplImage* pbBoundarySurface;
  	IplImage* pbBoundaryDisparity;
  	IplImage* bdOwnershipFlow;
  	IplImage* bdOwnershipSurface;

  	if (flowFLAG){
    		pbBoundaryFlow  = cvCloneImage(edgeGrad);
    		bdOwnershipFlow = cvCloneImage(edgeGrad); 
		cvSetZero(bdOwnershipFlow); 
    		calcPbBoundaryWtFlow(	pbBoundaryFlow, 
							edgeOri, 
							bdOwnershipFlow, 
							flowU, 
							flowV, 
							numOri);   
  	}

  	if (surfaceFLAG){
		IplImage* img_float = cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, 3);  
    		cvScale(img, img_float);
    		IplImage* img_c1c2c3 = cvCreateImage(cvSize(width, height), IPL_DEPTH_32F, 3);
    		bgrToc1c2c3(img_float, img_c1c2c3);

    		bdOwnershipSurface   = cvCloneImage(edgeGrad); 
		cvSetZero(bdOwnershipSurface);
    		pbBoundarySurface    = cvCloneImage(edgeGrad);
    		calcPbBoundaryWtSurface( img_c1c2c3, 
							pbBoundarySurface, 
							edgeOri, 
							bdOwnershipSurface, 
							groundColorHist_3D, 
							numOri, 
							nL, na, nb);
		cvReleaseImage(&img_c1c2c3);	
    		cvReleaseImage(&img_float);
  	}

  	if (disparityFLAG){
		pbBoundaryDisparity = cvCloneImage(edgeGrad);
		calcPbBoundaryWtDisparity( 	pbBoundaryDisparity,
								edgeOri,	
								dispMap,
								numOri);
	}

  // final probabilistic boundary!
  	if (flowFLAG && surfaceFLAG){
    		pbBoundary = cvCloneImage(edgeGrad);
    		cvMax(pbBoundarySurface, pbBoundaryFlow, pbBoundary);    
    		cvReleaseImage(&pbBoundarySurface);
    		cvReleaseImage(&pbBoundaryFlow);
    		cvReleaseImage(&bdOwnershipSurface);
    		cvReleaseImage(&bdOwnershipFlow);
	}
  	else if(disparityFLAG && surfaceFLAG){
		pbBoundary = cvCloneImage(edgeGrad);
    		cvMax(pbBoundarySurface, pbBoundaryDisparity, pbBoundary);    
    		cvReleaseImage(&pbBoundarySurface);
    		cvReleaseImage(&pbBoundaryDisparity);
  	}
  
  	if (flowFLAG && !surfaceFLAG){
    		pbBoundary = cvCloneImage(edgeGrad);
    		cvCopy(pbBoundaryFlow, pbBoundary);
    		cvReleaseImage(&pbBoundaryFlow);
    		cvReleaseImage(&bdOwnershipFlow);
  	}
  	else if(disparityFLAG && !surfaceFLAG){
		pbBoundary = cvCloneImage(edgeGrad);
    		cvCopy(pbBoundaryDisparity, pbBoundary);
    		cvReleaseImage(&pbBoundaryDisparity);
	}
  
  	if (!flowFLAG && !disparityFLAG && surfaceFLAG){
    		pbBoundary = cvCloneImage(edgeGrad);
    		cvCopy(pbBoundarySurface, pbBoundary);
    		cvReleaseImage(&pbBoundarySurface);
    		cvReleaseImage(&bdOwnershipSurface);
  	}
  
  	if(!flowFLAG && !surfaceFLAG && !disparityFLAG){
    		pbBoundary = cvCloneImage(edgeGrad);
  	}
   	return 0; 
}


//2)
int segLayer::generatePbBoundary(IplImage* mask){
	if(mask == NULL || mask->nChannels > 2){ 
		fprintf(stderr,"mask is empty or is not a gray image");
		exit(1);
	}
	pbBoundary  = cvCloneImage(edgeGrad);
    	IplImage* bdOwnership_tmp = cvCloneImage(edgeGrad); 
	cvSetZero(bdOwnership_tmp); 
    	calcPbBoundaryWtObjMask(	pbBoundary, 
						edgeOri, 
						mask, 
						bdOwnership_tmp, 
						numOri);  	
	cvReleaseImage(&bdOwnership_tmp);
}



//----------------------------------**
//Fixation Strategy
//1)
int segLayer::selectFixPt_interactive(){
 	while(true){ 
		cvShowImage("Select a fixation point on this image",img);
		cvSetMouseCallback("Select a fixation point on this image", &selectFixPt, &currFixPt);
  		cvWaitKey();	
		if (currFixPt.x < width && currFixPt.y < height){
			break;
		}
	}
	IplImage* tmpImg = cvCloneImage(img);
  	cvCircle(tmpImg, currFixPt, 5, CV_RGB(0, 255, 0), -1);
	cvShowImage("Select a fixation point on this image",tmpImg);
	cvReleaseImage(&tmpImg);

  	//add to the list
  	fixPts.push_back(currFixPt);
}

//3)
int segLayer::readFixPts(char *fixTxtFile){
  return 0;
}

//4)
int segLayer::assignFixPt(int x, int y){
	currFixPt.x = x;
	currFixPt.y =y;
	fixPts.push_back(currFixPt);
	return 0;
}

//5)
int segLayer::getNumFixPts(){
	return fixPts.size();
}

//6)
int segLayer::getFixPt(CvPoint& fixPt, int ind){
	if (ind < fixPts.size()){
		fixPt.x = fixPts[ind].x;
		fixPt.y = fixPts[ind].y;		
	}
	else{
		fprintf(stderr,"\n getFixPt(): error index out of the range of possible fixation points"); 
	}
	return 0;
}

CvPoint segLayer::getFixPt(int ind){
	if (ind < fixPts.size()){
		return fixPts[ind];				
	}
	else{
		fprintf(stderr,"\n getFixPt(): error index out of the range of possible fixation points"); 
	}
}

//-------------------------------**
//Segmentation
//1)
int segLayer::segmentAllFixs(){  
  if (storage == NULL){
    fprintf(stdout,"\n allocate memory before segmenting"); fflush(stdout);
    return 1;
  }

  IplImage* fgMap_uchar       = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U,   1);
  cvSetZero(fgMap_uchar);
  
  double* pb_ptr 		= (double*)malloc(sizeof(double)*width*height);
  double* ori_ptr 		= (double*)malloc(sizeof(double)*width*height);
  unsigned char* fg_ptr	= (unsigned char*)malloc(sizeof(unsigned char)*width*height);

  for(int y=0; y<height; y++)
	for(int x=0; x<width; x++){
		*(pb_ptr + width*y + x)  = CV_IMAGE_ELEM(pbBoundary, float, y, x);
		*(ori_ptr + width*y + x) = CV_IMAGE_ELEM(edgeOri, float, y, x);
		*(fg_ptr + width*y + x)  = 0;
	}

  for(int i=0; i< fixPts.size(); i++){
	 if (fixPts[i].x >= width || fixPts[i].y >= height){
		fprintf(stdout,"\n %dth fixation point outside of the image!",i+1); fflush(stdout);
    		continue;	
  	}
    	printf("\n segmenting fixNo.%d",i+1);
    	segFixatedRegion(	pb_ptr, 
				 	ori_ptr,
				 	fixPts[i].x,
				 	fixPts[i].y,
					width,
					height,
					fg_ptr);
	 for(int y=0; y<height; y++)
		for(int x=0; x<width; x++){
   			CV_IMAGE_ELEM(fgMap_uchar, unsigned char, y, x) = *(fg_ptr + width*y + x);
		}
    
    //store the contour!
    int countWtColor   = cvCountNonZero(fgMap_uchar);

    // extract the exterior contour and store it!
    CvSeq* first_contour;
    int Nc = cvFindContours(
			    fgMap_uchar,
			    storage,
			    &first_contour,
			    sizeof(CvContour),
			    CV_RETR_EXTERNAL);
    if (Nc == 1){
      allContours.push_back(first_contour);
    }
  }

  //release
  cvReleaseImage(&fgMap_uchar);  
  free(pb_ptr);
  free(ori_ptr);
  free(fg_ptr);
  return 0;  
}

//2)
int segLayer::segmentCurrFix(){  
  if (storage == NULL){
    fprintf(stdout,"\n allocate memory before segmenting"); fflush(stdout);
    return 1;
  }
  if (currFixPt.x >= width || currFixPt.y >= height){
	fprintf(stdout,"\n the fixation point outside of the image!"); fflush(stdout);
    	return 1;	
  }
  IplImage* fgMap_uchar       = cvCreateImage(cvSize(width, height), IPL_DEPTH_8U, 1);
  cvSetZero(fgMap_uchar);

  double* pb_ptr 		= (double*)malloc(sizeof(double)*width*height);
  double* ori_ptr 		= (double*)malloc(sizeof(double)*width*height);
  unsigned char* fg_ptr	= (unsigned char*)malloc(sizeof(unsigned char)*width*height);

  for(int y=0; y<height; y++)
	for(int x=0; x<width; x++){
		*(pb_ptr + width*y + x)  = CV_IMAGE_ELEM(pbBoundary, float, y, x);
		*(ori_ptr + width*y + x) = CV_IMAGE_ELEM(edgeOri, float, y, x);
		*(fg_ptr + width*y + x)  = 0;
	}

  printf("\n segmenting for the fixation point = (%d, %d)\n",currFixPt.x, currFixPt.y);
  
  
  segFixatedRegion_Cart_Wfix(	pb_ptr, 
				ori_ptr,
				currFixPt.x,
				currFixPt.y,
				width,
				height,
				fixX,
				fixY,
				fixNum,
				fixWeight,
				fg_ptr,
				cenX,
				cenY); 
	

  //printf("segmentation completed\n");
  for(int y=0; y<height; y++)
	for(int x=0; x<width; x++){
   		CV_IMAGE_ELEM(fgMap_uchar, unsigned char, y, x) = *(fg_ptr + width*y + x);
	}
    
  //debug the output labels;
  cvShowImage("labels", fgMap_uchar);
   //store the contour!
  int countWtColor   = cvCountNonZero(fgMap_uchar);

	
  // extract the exterior contour and store it!
  allContours.clear();
  CvSeq* first_contour;
  int Nc = cvFindContours(
			  fgMap_uchar,
			  storage,
			  &first_contour,
			  sizeof(CvContour),
			  CV_RETR_EXTERNAL);
			  
  for (; first_contour != 0; first_contour = first_contour->h_next){
    allContours.push_back(first_contour);
  }

  //release
  cvReleaseImage(&fgMap_uchar);
	 
  free(pb_ptr);
  free(ori_ptr);
  free(fg_ptr);
  return 0;  
}

//3)
int segLayer::allocateMemForContours(){
  storage = cvCreateMemStorage();
}
//4)
int segLayer::deallocateMemForContours(){
  if (storage != NULL)
  	cvReleaseMemStorage(&storage);
}
//5)
int segLayer::clearAllContours(){
	allContours.clear();
}


//--------------------------------**
// Saving regions
//1)
int segLayer::saveRegions(char* prefixTag){
	IplImage* tmp = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
	char segName[100];
     int numContours = allContours.size();
	for(int i=0; i<numContours; i++){
		cvSetZero(tmp);
		cvDrawContours(
		 			tmp,
		 			allContours[i],
		 			CV_RGB(255,255,255),
			 		CV_RGB(255,255,255),
		 			-1,
		  			-1);
		sprintf(segName,"%s_region_%d.png",prefixTag, i+1);
		cvSaveImage(segName, tmp);		
	}
	cvReleaseImage(&tmp);
	return 0;
}

//2)
int segLayer::saveEdgeMap(char *prefixTag){
  IplImage* tmpImg = cvCreateImage(cvGetSize(edgeGrad), IPL_DEPTH_8U, 1);
  cvScale(edgeGrad, tmpImg, 255);	
  if (prefixTag == NULL){
    char edgeMapName_noprefix[100];
    sprintf(edgeMapName_noprefix,"gradientMap.png");
    cvSaveImage(edgeMapName_noprefix, tmpImg);
  }
  else{
    char edgeMapName[100];
    sprintf(edgeMapName,"%s_gradientMap.png",prefixTag);
    cvSaveImage(edgeMapName, tmpImg);    
  }
  cvReleaseImage(&tmpImg);
  return 0;  
}

//3)
int segLayer::savePbBoundary(char *prefixTag){
  IplImage* tmpImg = cvCreateImage(cvGetSize(pbBoundary), IPL_DEPTH_8U, 1);
  cvScale(pbBoundary, tmpImg, 255);	
  if (prefixTag == NULL){
    char pbMapName_noprefix[100];
    sprintf(pbMapName_noprefix,"pbBoundary.png");
    cvSaveImage(pbMapName_noprefix, tmpImg);
  }
  else{
    char pbMapName[100];
    sprintf(pbMapName,"%s_pbBoundary.png",prefixTag);
    cvSaveImage(pbMapName, tmpImg);    
  }
  cvReleaseImage(&tmpImg);
  return 0;  
}

//TCL -- only 1 region considered, save MASK, RGB
int segLayer::saveRegionsnRGB(char* prefixTag){
	IplImage* tmp = cvCreateImage(cvSize(width,height), IPL_DEPTH_8U, 1);
	char segName[100];
	char imgName[100];
	char dispName[100];
    int numContours = allContours.size();
	for(int i=0; i<1; i++){
		cvSetZero(tmp);
		printf("segmentation completed\n");
		cvDrawContours(
		 			tmp,
		 			allContours[numContours-1],
		 			CV_RGB(255,255,255),
			 		CV_RGB(255,255,255),
		 			-1,
		  			-1);
		sprintf(segName,"%s_MASK.png",prefixTag);	
		cvSaveImage(segName, tmp);	
		sprintf(imgName, "%s_RGB.png",prefixTag);	
		cvSaveImage(imgName, img);
		sprintf(dispName, "%s_DEPTH.xml",prefixTag);
		cvSave(dispName, dispMap);
/*
		//quick test
		IplImage* dispMap2 = (IplImage*)cvLoad(dispName);
		cvNamedWindow("test");
		cvShowImage("test", dispMap2);
		cvWaitKey(-1);
*/

	}
	
	cvReleaseImage(&tmp);
	

	return 0;
}


//-----------------------------**
//Display:
//1) 
int segLayer::displayImg(int delay){ 
  if (img == NULL){
    fprintf(stdout, "\nWarning: no image assigned yet"); fflush(stdout);
    return 1;
  }
  cvShowImage("RGB image",img);
  cvWaitKey(delay);
  return 0;
}

int segLayer::displayMask(int delay){ 
  if (img == NULL){
    fprintf(stdout, "\nWarning: no mask assigned yet"); fflush(stdout);
    return 1;
  }
  cvShowImage("object mask",mask);
  cvWaitKey(delay);
  return 0;
}

//2)
int segLayer::displayEdge(int delay){
  if (edgeGrad == NULL){
    fprintf(stdout, "\nWarning: no edge assigned yet"); fflush(stdout);
    return 1;
  }
  cvShowImage("edge", edgeGrad);
  cvWaitKey(delay);
  return 0;
}
//3)
int segLayer::displayPbBoundary(int delay){
  if (pbBoundary == NULL){
    fprintf(stdout, "\nWarning: probabilistic boundary not calculated yet"); fflush(stdout);
    return 1;
  }
  cvShowImage("final probabilistic boundary", pbBoundary);
  cvWaitKey(delay);
  return 0;  
}
//4)
int segLayer::displayFlowMag(int delay){
  IplImage* flowU_tmp = cvCloneImage(flowU);
  IplImage* flowV_tmp = cvCloneImage(flowV);
  IplImage* flowMag   = cvCloneImage(flowU); 
  IplImage* flowMag_uchar = cvCreateImage(cvGetSize(flowU), IPL_DEPTH_8U, 1);
  cvSetZero(flowMag);

  cvPow(flowU_tmp, flowU_tmp, 2);
  cvPow(flowV_tmp, flowV_tmp, 2);
  cvAdd(flowU_tmp, flowV_tmp, flowMag);
  cvPow(flowMag,flowMag, 0.5);

  double minVal, maxVal;
  cvMinMaxLoc(flowMag, &minVal, &maxVal);
  cvScale(flowMag, flowMag_uchar, 255/(maxVal-minVal), -minVal);
  cvShowImage("flow magnitude", flowMag_uchar);
  cvWaitKey(delay);
  
  cvReleaseImage(&flowU_tmp);
  cvReleaseImage(&flowV_tmp);
  cvReleaseImage(&flowMag);
  cvReleaseImage(&flowMag_uchar);
  
  return 0;
}
//5)
int segLayer::displayCurrSegs(int delay){
  IplImage* tmp      = cvCloneImage(img);
  int len = allContours.size();
  cvDrawContours(
		 tmp,
		 allContours[len-1],
		 CV_RGB(0,255,0),
		 CV_RGB(255,255,255),
		 -1,
		  5);
  cvShowImage("segmented object mask", tmp);
  cvWaitKey(delay);
  cvReleaseImage(&tmp);
  return 0;  
}

//6)
int segLayer::displayCurrFixPt(int delay){
  IplImage* tmpImg = cvCloneImage(img);
  cvCircle(tmpImg, currFixPt, 5, CV_RGB(0, 255, 0), -1);
  cvShowImage("image with fixation", tmpImg);
  cvWaitKey(delay);
  cvReleaseImage(&tmpImg);
  return 0;
}

//7)
int segLayer::displayAllFixPts(int delay){
  IplImage* tmpImg = cvCloneImage(img);  
  for(int i=0; i<fixPts.size(); i++){
    cvCircle(tmpImg, fixPts[i], 5, CV_RGB(0, 255, 0), -1);
  }
  cvShowImage("image with fixations", tmpImg);
  cvWaitKey(delay);
  cvReleaseImage(&tmpImg);
  return 0;
}
//8)
int segLayer::saveFixPtsImage(char* prefixTag){
	fprintf(stdout, "\n inside saveFixPtsImage function : number of fixation points= %d", fixPts.size()); fflush(stdout);
 	IplImage* tmpImg = cvCloneImage(img);  
  	for(int i=0; i<fixPts.size(); i++){
    	cvCircle(tmpImg, fixPts[i], 5, CV_RGB(0, 255, 0), -1);
  	}
  	if (prefixTag == NULL){
    	char pbMapName_noprefix[100];
    	sprintf(pbMapName_noprefix,"fixPts.png");
    	cvSaveImage(pbMapName_noprefix, tmpImg);
  	}
  	else{
    	char pbMapName[100];
    	sprintf(pbMapName,"%s_fixPts.png",prefixTag);
    	cvSaveImage(pbMapName, tmpImg);    
  	}
  	//save image!
  	cvReleaseImage(&tmpImg);
  	return 0;  
}




