#include <ros/ros.h>
#include <image_transport/image_transport.h>
#include <cv_bridge/cv_bridge.h>
#include <sensor_msgs/image_encodings.h>
#include <opencv2/imgproc/imgproc.hpp>
#include <opencv2/highgui/highgui.hpp>
#include "graph.h"
#include <segmentationLayer.h>

namespace enc = sensor_msgs::image_encodings;

static const char WINDOW[] = "Image window";

class ImageConverter
{
  ros::NodeHandle nh_;
  image_transport::ImageTransport it_;
  image_transport::Subscriber image_sub_;
  image_transport::Publisher image_pub_;
//for revised segmentation code
  segLayer* segL;
  int proc_width, proc_height;
  
  int loopCt;
  
public:
  ImageConverter()
    : it_(nh_)
  {
    image_pub_ = it_.advertise("out", 1);
    image_sub_ = it_.subscribe("/camera/rgb/image_color", 1, &ImageConverter::imageCb, this);
    segL = new segLayer();
    proc_width = 320*2; // default processing size
    proc_height = 240*2;
    loopCt = 0;
    cv::namedWindow(WINDOW);
    cvNamedWindow("mask_img", 1);
  }

  ~ImageConverter()
  {
    delete segL;
    cv::destroyWindow(WINDOW);
  }

  void imageCb(const sensor_msgs::ImageConstPtr& msg)
  {
    cv_bridge::CvImagePtr cv_ptr;
    try
    {
      cv_ptr = cv_bridge::toCvCopy(msg, enc::BGR8);
    }
    catch (cv_bridge::Exception& e)
    {
      ROS_ERROR("cv_bridge exception: %s", e.what());
      return;
    }
    
    //convert to IplImage
    //printf("1\n");
    IplImage ipl_img = cv_ptr->image;
    IplImage* ipl_img_small = cvCreateImage(cvSize(proc_width, proc_height), IPL_DEPTH_8U, 3);
    
	cvResize(&ipl_img, ipl_img_small);
	
    segL->setImage(ipl_img_small);
    //printf("2\n");
    //segL->edgeCGTG();
    segL->edgeSobel();
    //printf("3\n");
	segL->generatePbBoundary();
	//printf("4\n");
	if(loopCt == 0)
	    segL->selectFixPt_interactive();
	//printf("5\n");
	segL->fixationToParticles(loopCt);
	//printf("6\n");
	segL->allocateMemForContours();
	//printf("7\n");
    segL->segmentCurrFix();
    
    
    //printf("8\n");
    IplImage* img_mask = segL->getCurrMask();
//    if (cv_ptr->image.rows > 60 && cv_ptr->image.cols > 60)
//      cv::circle(cv_ptr->image, cv::Point(50, 50), 10, CV_RGB(255,0,0));

	cvShowImage("mask_img", img_mask);
    cvWaitKey(30);
    
    segL->updateTargetDis();
    image_pub_.publish(cv_ptr->toImageMsg());
    cvReleaseImage(&img_mask);
    loopCt ++;
  }
};

int main(int argc, char** argv)
{
    
  ros::init(argc, argv, "image_converter");
  ImageConverter ic;
  ros::spin();
  return 0;
}

